@@ -18,7 +18,7 @@
 // Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
 // Copyright (C) 2007 Jeff Muizelaar <jeff@infidigm.net>
-// Copyright (C) 2008-2011, 2016-2018, 2021 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008-2011, 2016-2018, 2022 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2011 Kenji Uno <ku@digitaldolphins.jp>
 // Copyright (C) 2012, 2013 Fabio D'Urso <fabiodurso@hotmail.it>
 // Copyright (C) 2012, 2017 Adrian Johnson <ajohnson@redneon.com>
@@ -121,9 +121,9 @@ void formatDoubleSmallAware(double x, char *buf, int bufSize, int prec, bool tri
 
 //------------------------------------------------------------------------
 
-GooString *GooString::format(const char *fmt, ...)
+std::unique_ptr<GooString> GooString::format(const char *fmt, ...)
 {
-    auto *s = new GooString();
+    auto s = std::make_unique<GooString>();
 
     va_list argList;
     va_start(argList, fmt);
@@ -133,9 +133,9 @@ GooString *GooString::format(const char *fmt, ...)
     return s;
 }
 
-GooString *GooString::formatv(const char *fmt, va_list argList)
+std::unique_ptr<GooString> GooString::formatv(const char *fmt, va_list argList)
 {
-    auto *s = new GooString();
+    auto s = std::make_unique<GooString>();
 
     s->appendfv(fmt, argList);
 
@@ -456,8 +456,9 @@ GooString *GooString::appendfv(const char *fmt, va_list argList)
             append('}');
 
         } else {
-            for (p1 = p0 + 1; *p1 && *p1 != '{' && *p1 != '}'; ++p1)
+            for (p1 = p0 + 1; *p1 && *p1 != '{' && *p1 != '}'; ++p1) {
                 ;
+            }
             append(p0, p1 - p0);
             p0 = p1;
         }
@@ -589,13 +590,24 @@ void formatDoubleSmallAware(double x, char *buf, int bufSize, int prec, bool tri
 
 GooString *GooString::lowerCase()
 {
-    for (auto &c : *this) {
+    lowerCase(*this);
+    return this;
+}
+
+void GooString::lowerCase(std::string &s)
+{
+    for (auto &c : s) {
         if (std::isupper(c)) {
             c = std::tolower(c);
         }
     }
+}
 
-    return this;
+std::string GooString::toLowerCase(const std::string &s)
+{
+    std::string newString = s;
+    lowerCase(newString);
+    return s;
 }
 
 void GooString::prependUnicodeMarker()
@@ -605,24 +617,12 @@ void GooString::prependUnicodeMarker()
 
 bool GooString::startsWith(const char *prefix) const
 {
-    const auto len = size();
-    const auto prefixLen = std::strlen(prefix);
-
-    if (len < prefixLen)
-        return false;
-
-    return static_cast<const std::string &>(*this).compare(0, prefixLen, prefix) == 0;
+    return startsWith(toStr(), prefix);
 }
 
 bool GooString::endsWith(const char *suffix) const
 {
-    const auto len = size();
-    const auto suffixLen = std::strlen(suffix);
-
-    if (len < suffixLen)
-        return false;
-
-    return static_cast<const std::string &>(*this).compare(len - suffixLen, suffixLen, suffix) == 0;
+    return endsWith(toStr(), suffix);
 }
 
 GooString *GooString::sanitizedName(bool psmode) const
