@@ -121,8 +121,9 @@ void PdfInspector::on_file_activated(GtkWidget *widget, PdfInspector *inspector)
     gchar *file_name;
 
     file_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
-    if (file_name)
+    if (file_name) {
         inspector->load(file_name);
+    }
 
     g_free(file_name);
 }
@@ -142,8 +143,9 @@ void PdfInspector::on_selection_changed(GtkTreeSelection *selection, PdfInspecto
         gtk_tree_model_get(model, &iter, OP_STRING, &op, -1);
     }
 
-    if (op == nullptr)
+    if (op == nullptr) {
         return;
+    }
 
     for (i = 0; i < G_N_ELEMENTS(op_mapping); i++) {
 
@@ -276,17 +278,18 @@ int main(int argc, char *argv[])
     globalParams->setProfileCommands(true);
     globalParams->setPrintCommands(true);
 
-    if (argc == 2)
+    if (argc == 2) {
         file_name = argv[1];
-    else if (argc > 2) {
+    } else if (argc > 2) {
         fprintf(stderr, "usage: %s [PDF-FILE]\n", argv[0]);
         return -1;
     }
 
     inspector = new PdfInspector();
 
-    if (file_name)
+    if (file_name) {
         inspector->set_file_name(file_name);
+    }
 
     inspector->run();
 
