@@ -15,6 +15,7 @@
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2019 Marek Kasik <mkasik@redhat.com>
 // Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2022 crt <chluo@cse.cuhk.edu.hk>
 //
 //========================================================================
 
@@ -197,6 +198,14 @@ int main(int argc, char *argv[])
     Object ocObj;
     if (docs.size() >= 1) {
         Object catObj = docs[0]->getXRef()->getCatalog();
+        if (!catObj.isDict()) {
+            fclose(f);
+            delete yRef;
+            delete countRef;
+            delete outStr;
+            error(errSyntaxError, -1, "XRef's Catalog is not a dictionary.");
+            return -1;
+        }
         Dict *catDict = catObj.getDict();
         intents = catDict->lookup("OutputIntents");
         afObj = catDict->lookupNF("AcroForm").copy();
@@ -275,8 +284,9 @@ int main(int argc, char *argv[])
             }
 
             const PDFRectangle *cropBox = nullptr;
-            if (docs[i]->getCatalog()->getPage(j)->isCropped())
+            if (docs[i]->getCatalog()->getPage(j)->isCropped()) {
                 cropBox = docs[i]->getCatalog()->getPage(j)->getCropBox();
+            }
             docs[i]->replacePageDict(j, docs[i]->getCatalog()->getPage(j)->getRotate(), docs[i]->getCatalog()->getPage(j)->getMediaBox(), cropBox);
             Ref *refPage = docs[i]->getCatalog()->getPageRef(j);
             Object page = docs[i]->getXRef()->fetch(*refPage);
@@ -294,6 +304,14 @@ int main(int argc, char *argv[])
             }
         }
         Object pageCatObj = docs[i]->getXRef()->getCatalog();
+        if (!pageCatObj.isDict()) {
+            fclose(f);
+            delete yRef;
+            delete countRef;
+            delete outStr;
+            error(errSyntaxError, -1, "XRef's Catalog is not a dictionary.");
+            return -1;
+        }
         Dict *pageCatDict = pageCatObj.getDict();
         Object pageNames = pageCatDict->lookup("Names");
         if (!pageNames.isNull() && pageNames.isDict()) {
@@ -350,8 +368,9 @@ int main(int argc, char *argv[])
     yRef->add(rootNum + 1, 0, outStr->getPos(), true);
     outStr->printf("%d 0 obj\n", rootNum + 1);
     outStr->printf("<< /Type /Pages /Kids [");
-    for (j = 0; j < (int)pages.size(); j++)
+    for (j = 0; j < (int)pages.size(); j++) {
         outStr->printf(" %d 0 R", rootNum + j + 2);
+    }
     outStr->printf(" ] /Count %zd >>\nendobj\n", pages.size());
     objectsCount++;
 
@@ -361,8 +380,9 @@ int main(int argc, char *argv[])
         outStr->printf("<< ");
         Dict *pageDict = pages[i].getDict();
         for (j = 0; j < pageDict->getLength(); j++) {
-            if (j > 0)
+            if (j > 0) {
                 outStr->printf(" ");
+            }
             const char *key = pageDict->getKey(j);
             Object value = pageDict->getValNF(j).copy();
             if (strcmp(key, "Parent") == 0) {
@@ -389,7 +409,8 @@ int main(int argc, char *argv[])
     fclose(f);
     delete yRef;
     delete countRef;
-    for (i = 0; i < (int)docs.size(); i++)
+    for (i = 0; i < (int)docs.size(); i++) {
         delete docs[i];
+    }
     return 0;
 }
